@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -31,6 +31,7 @@ public:
 	void getOption(const s8 *key, u32 &val);
 	void getOption(const s8 *key, bool &val);
 	bool parseArgs(s32 argc, s8 **argv);
+	s8** getShellCommand() { return mShellCommand; }
 
 private:
 	void checkConfigFile(const s8 *name);
@@ -47,6 +48,7 @@ private:
 
 	s8 *mConfigBuf;
 	OptionEntry *mConfigEntrys;
+	s8 **mShellCommand;
 };
 
 #endif
