@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -19,17 +19,21 @@
  */
 
 #include <unistd.h>
-#include <signal.h>
 #include <fcntl.h>
-#include <string.h>
-#include <locale.h>
-#include <langinfo.h>
-#include <sys/epoll.h>
+#include "config.h"
 #include "fbio.h"
 
-#define NR_EPOLL_FDS 10
 #define NR_FDS 32
 
+#ifdef HAVE_EPOLL
+#include <sys/epoll.h>
+#define NR_EPOLL_FDS 10
+s32 epollFd;
+#else
+static fd_set fds;
+static u32 maxfd = 0;
+#endif
+
 static IoPipe *ioPipeMap[NR_FDS];
 
 IoDispatcher *IoDispatcher::createInstance()
@@ -39,8 +43,12 @@ IoDispatcher *IoDispatcher::createInstance()
 
 FbIoDispatcher::FbIoDispatcher()
 {
-	mEpollFd = epoll_create(NR_EPOLL_FDS);
-	fcntl(mEpollFd, F_SETFD, fcntl(mEpollFd, F_GETFD) | FD_CLOEXEC);
+#ifdef HAVE_EPOLL
+	epollFd = epoll_create(NR_EPOLL_FDS);
+	fcntl(epollFd, F_SETFD, fcntl(epollFd, F_GETFD) | FD_CLOEXEC);
+#else
+	FD_ZERO(&fds);
+#endif
 }
 
 FbIoDispatcher::~FbIoDispatcher()
@@ -49,7 +57,9 @@ FbIoDispatcher::~FbIoDispatcher()
 		if (ioPipeMap[i]) delete ioPipeMap[i];
 	}
 
-	close(mEpollFd);
+#ifdef HAVE_EPOLL
+	close(epollFd);
+#endif
 }
 
 void FbIoDispatcher::addIoSource(IoPipe *src, bool isread)
@@ -57,10 +67,15 @@ void FbIoDispatcher::addIoSource(IoPipe *src, bool isread)
 	if (src->fd() >= NR_FDS) return;
 	ioPipeMap[src->fd()] = src;
 
+#ifdef HAVE_EPOLL
 	epoll_event ev;
 	ev.data.fd = src->fd();
 	ev.events = (isread ? EPOLLIN : EPOLLOUT);
-	epoll_ctl(mEpollFd, EPOLL_CTL_ADD, src->fd(), &ev);
+	epoll_ctl(epollFd, EPOLL_CTL_ADD, src->fd(), &ev);
+#else
+	FD_SET(src->fd(), &fds);
+	if (src->fd() > maxfd) maxfd = src->fd();
+#endif
 }
 
 void FbIoDispatcher::removeIoSource(IoPipe *src, bool isread)
@@ -68,17 +83,21 @@ void FbIoDispatcher::removeIoSource(IoPipe *src, bool isread)
 	if (src->fd() >= NR_FDS) return;
 	ioPipeMap[src->fd()] = 0;
 
+#ifdef HAVE_EPOLL
 	epoll_event ev;
 	ev.data.fd = src->fd();
 	ev.events = (isread ? EPOLLIN : EPOLLOUT);
-	epoll_ctl(mEpollFd, EPOLL_CTL_DEL, src->fd(), &ev);
-
+	epoll_ctl(epollFd, EPOLL_CTL_DEL, src->fd(), &ev);
+#else
+	FD_CLR(src->fd(), &fds);
+#endif
 }
 
 void FbIoDispatcher::poll()
 {
+#ifdef HAVE_EPOLL
 	epoll_event evs[NR_EPOLL_FDS];
-	s32 nfds = epoll_wait(mEpollFd, evs, NR_EPOLL_FDS, -1);
+	s32 nfds = epoll_wait(epollFd, evs, NR_EPOLL_FDS, -1);
 
 	for (s32 i = 0; i < nfds; i++) {
 		IoPipe *src = ioPipeMap[evs[i].data.fd];
@@ -96,4 +115,18 @@ void FbIoDispatcher::poll()
 			delete src;
 		}
 	}
+#else
+	fd_set rfds = fds;
+	s32 num = select(maxfd + 1, &rfds, 0, 0, 0);
+	if (num <= 0) return;
+
+	for (u32 i = 0; i <= maxfd; i++) {
+		if (FD_ISSET(i, &rfds)) {
+			if (ioPipeMap[i]) {
+				ioPipeMap[i]->ready(true);
+			}
+			if (!--num) break;
+		}
+	}
+#endif
 }
