@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -26,6 +26,20 @@
 #include "fbconfig.h"
 
 #define OFFSET(TYPE, MEMBER) ((size_t)(&(((TYPE *)0)->MEMBER)))
+#define SUBS(a, b) ((a) > (b) ? (a) - (b) : (b) - (a))
+#define MIN(a, b) ((a) < (b) ? (a) : (b))
+
+static FcCharSet *unicodeMap;
+static FcFontSet *fontList;
+ 
+static FT_Library ftlib;
+static FT_Face *fontFaces;
+static u32 *fontFlags;
+
+static Font::Glyph **glyphCache;
+static bool *glyphCacheInited;
+
+static void openFont(u32 index);
 
 DEFINE_INSTANCE(Font)
 
@@ -33,14 +47,14 @@ Font *Font::createInstance()
 {
 	FcInit();
 
-    s8 buf[64];
-    Config::instance()->getOption("font-names", buf, sizeof(buf));
+	s8 buf[64];
+	Config::instance()->getOption("font-names", buf, sizeof(buf));
 
-    FcPattern *pat = FcNameParse((FcChar8 *)(*buf ? buf : "mono"));
+	FcPattern *pat = FcNameParse((FcChar8 *)(*buf ? buf : "mono"));
 
-    u32 pixel_size = 12;
-    Config::instance()->getOption("font-size", pixel_size);
-    FcPatternAddDouble(pat, FC_PIXEL_SIZE, (double)pixel_size);
+	u32 pixel_size = 12;
+	Config::instance()->getOption("font-size", pixel_size);
+	FcPatternAddDouble(pat, FC_PIXEL_SIZE, (double)pixel_size);
 
 	FcPatternAddString(pat, FC_LANG, (FcChar8 *)"en");
 
@@ -48,87 +62,102 @@ Font *Font::createInstance()
 	FcDefaultSubstitute(pat);
 
 	FcResult result;
-	FcCharSet *cs;
-	FcFontSet *fs = FcFontSort(NULL, pat, FcTrue, &cs, &result);
+	FcFontSet *fs = FcFontSort(NULL, pat, FcTrue, &unicodeMap, &result);
 
-	FontRec *fonts = 0;
-	u32 index = 0;
 	if (fs) {
-		fonts = new FontRec[fs->nfont];
-	
-		for (s32 i = 0; i < fs->nfont; i++) {
+		fontList = FcFontSetCreate();
+
+		FcObjectSet *family = FcObjectSetCreate();
+		FcObjectSetAdd(family, FC_FAMILY);
+
+		for (u32 i = 0; i < fs->nfont; i++) {
 			FcPattern *font = FcFontRenderPrepare(NULL, pat, fs->fonts[i]);
-			if (font) {
-				fonts[index].face = 0;
-				fonts[index++].pattern = font;
+			if (!font) continue;
+
+			bool same = false;
+			for (u32 j = 0; j < fontList->nfont; j++) {
+				if (FcPatternEqualSubset(fontList->fonts[j], font, family)) {
+					same = true;
+					break;
+				}
+			}
+
+			if (same) {
+				FcPatternDestroy(font);
+			} else {
+				FcFontSetAdd(fontList, font);
 			}
 		}
+
+		FcObjectSetDestroy(family);
 	}
 
 	FcPatternDestroy(pat);
 	if (fs) FcFontSetDestroy(fs);
 
-	if (!index) {
-		if (cs) {
-			FcCharSetDestroy(cs);
-			delete[] fonts;
-		}
+	if (fontList && fontList->nfont) return new Font();
 
-		FcFini();
-		return 0;
-	}
-
-	return new Font(fonts, index, cs);
+	if (unicodeMap) FcCharSetDestroy(unicodeMap);
+	if (fontList) FcFontSetDestroy(fontList);
+	FcFini();
+	return 0;
 }
 
-static FT_Library ftlib = 0;
-static Font::Glyph **glyphCache;
-static bool *glyphCacheInited;
-
-Font::Font(FontRec *fonts, u32 num, void *unicover)
+Font::Font()
 {
-	mpFontList = fonts;
-	mFontNum = num;
-	mpUniCover = unicover;
-	mMonospace = false;
+	mHeight = mWidth = 0;
+
+	fontFaces = new FT_Face[fontList->nfont];
+	fontFlags = new u32[fontList->nfont];
+	memset(fontFaces, 0, sizeof(FT_Face) * fontList->nfont);
+
 	glyphCache = new Glyph *[256 * 256];
 	glyphCacheInited = new bool[256];
-	bzero(glyphCacheInited, sizeof(bool) * 256);
-
-	int spacing;
-	if (FcPatternGetInteger((FcPattern*)mpFontList[0].pattern, FC_SPACING, 0, &spacing) == FcResultMatch)
-		mMonospace = (spacing != FC_PROPORTIONAL);
+	memset(glyphCacheInited, 0, sizeof(bool) * 256);
 
 	FT_Init_FreeType(&ftlib);
-
 	openFont(0);
-	mHeight = ((FT_Face)mpFontList[0].face)->size->metrics.height >> 6;
-	mWidth = ((FT_Face)mpFontList[0].face)->size->metrics.max_advance >> 6;
 
-	if (!mMonospace) {
-		Glyph *glyph = getGlyph('a');
-		if (mWidth > glyph->advance * 2) mWidth /= 2;
+	FT_Face face = fontFaces[0];
+	if (face == (FT_Face)-1) return;
+
+	if (face->face_flags & FT_FACE_FLAG_SCALABLE) {
+		mHeight = face->size->metrics.height >> 6;
+		mWidth = face->size->metrics.max_advance >> 6;
+	} else if (face->num_fixed_sizes) {
+		double dsize;
+		FcPatternGetDouble(fontList->fonts[0], FC_PIXEL_SIZE, 0, &dsize);
+
+		FT_Bitmap_Size *sizes = face->available_sizes;
+		u32 index = 0, diffmin = (u32)-1;
+		for (u32 i = 0; i < face->num_fixed_sizes; i++) {
+			u32 diff = SUBS(sizes[i].size >> 6, (u32)dsize);
+			if (diff < diffmin ) {
+				index = i;
+				diffmin = diff;
+			}
+		}
+
+		mHeight = sizes[index].height;
+		mWidth = sizes[index].width;
+	}
+
+	if (!(face->face_flags & FT_FACE_FLAG_FIXED_WIDTH)) mWidth = MIN(mWidth, (mHeight + 1) / 2);
+
+	u32 width = 0;
+	Config::instance()->getOption("font-width", width);
+
+	if (width) {
+		s8 buf[64];
+		Config::instance()->getOption("font-width", buf, sizeof(buf));
+
+		if (buf[0] == '+' || buf[0] == '-') mWidth += (s32)width;
+		else mWidth = width;
 	}
 }
 
 Font::~Font()
 {
-	FcCharSetDestroy((FcCharSet*)mpUniCover);
-
-	for (u32 i = 0; i < mFontNum; i++) {	
-		FcPatternDestroy((FcPattern*)mpFontList[i].pattern);
-
-		FT_Face face = (FT_Face)mpFontList[i].face;
-		if (face > 0) {
-			FT_Done_Face(face);
-		}
-	}
-
-	FT_Done_FreeType(ftlib);
-	FcFini();
-
-	delete[] mpFontList;	
-
 	for (u32 i = 0; i < 256; i++) {
 		if (!glyphCacheInited[i]) continue;
 
@@ -138,16 +167,47 @@ Font::~Font()
 			}
 		}
 	}
-	
+
 	delete[] glyphCache;
 	delete[] glyphCacheInited;
+
+	for (u32 i = 0; i < fontList->nfont; i++) {
+		if (fontFaces[i] && fontFaces[i] != (FT_Face)-1) {
+			FT_Done_Face(fontFaces[i]);
+		}
+	}
+
+	delete[] fontFaces;
+	delete[] fontFlags;
+
+	FT_Done_FreeType(ftlib);
+	FcCharSetDestroy(unicodeMap);
+	FcFontSetDestroy(fontList);
+	FcFini();
 }
 
-void Font::openFont(u32 index)
+void Font::showInfo(bool verbose)
 {
-	if (index >= mFontNum) return;
+	if (!verbose) return;
 
-	FcPattern *pattern = (FcPattern*)mpFontList[index].pattern;
+	printf("[font] width: %dpx, height: %dpx, ordered list: ", mWidth, mHeight);
+
+	u32 index;
+	FcChar8 *family;
+	for (index = 0; index < fontList->nfont - 1; index++) {
+		FcPatternGetString(fontList->fonts[index], FC_FAMILY, 0, &family);
+		printf("%s, ", family);
+	}
+
+	FcPatternGetString(fontList->fonts[index], FC_FAMILY, 0, &family);
+	printf("%s\n", family);
+}
+
+static void openFont(u32 index)
+{
+	if (index >= fontList->nfont) return;
+
+	FcPattern *pattern = fontList->fonts[index];
 
 	FcChar8 *name = (FcChar8 *)"";
 	FcPatternGetString(pattern, FC_FILE, 0, &name);
@@ -157,7 +217,7 @@ void Font::openFont(u32 index)
 
 	FT_Face face;
 	if (FT_New_Face(ftlib, (const char *)name, id, &face)) {
-		mpFontList[index].face = (void *)-1;
+		fontFaces[index] = (FT_Face)-1;
 		return;
 	}
 
@@ -181,24 +241,24 @@ void Font::openFont(u32 index)
 
 		if (!hinting || hint_style == FC_HINT_NONE) {
 			load_flags |= FT_LOAD_NO_HINTING;
-		} else { 
+		} else {
 			load_flags |= FT_LOAD_TARGET_LIGHT;
 		}
 	} else {
 		load_flags |= FT_LOAD_TARGET_MONO;
 	}
 
-	mpFontList[index].face = face;
-	mpFontList[index].load_flags = load_flags;
+	fontFaces[index] = face;
+	fontFlags[index] = load_flags;
 }
 
-int Font::fontIndex(u32 unicode)
+static int fontIndex(u32 unicode)
 {
-	if (!FcCharSetHasChar((FcCharSet *)mpUniCover, (FcChar32)unicode)) return -1;
+	if (!FcCharSetHasChar(unicodeMap, unicode)) return -1;
 
 	FcCharSet *charset;
-	for (u32 i = 0; i < mFontNum; i++) {
-		FcPatternGetCharSet((FcPattern *)mpFontList[i].pattern, FC_CHARSET, 0, &charset);
+	for (u32 i = 0; i < fontList->nfont; i++) {
+		FcPatternGetCharSet(fontList->fonts[i], FC_CHARSET, 0, &charset);
 		if (FcCharSetHasChar(charset, unicode)) return i;
 	}
 
@@ -208,10 +268,10 @@ int Font::fontIndex(u32 unicode)
 Font::Glyph *Font::getGlyph(u32 unicode)
 {
 	if (unicode >= 256 * 256) return 0;
-	
+
 	if (!glyphCacheInited[unicode >> 8]) {
 		glyphCacheInited[unicode >> 8] = true;
-		bzero(&glyphCache[unicode & 0xff00], sizeof(Glyph *) * 256);
+		memset(&glyphCache[unicode & 0xff00], 0, sizeof(Glyph *) * 256);
 	}
 
 	if (glyphCache[unicode]) return glyphCache[unicode];
@@ -219,42 +279,40 @@ Font::Glyph *Font::getGlyph(u32 unicode)
 	int i = fontIndex(unicode);
 	if (i == -1) return 0;
 
-	if (!mpFontList[i].face) openFont(i);
-	if (mpFontList[i].face == (void *)-1) return 0;
+	if (!fontFaces[i]) openFont(i);
+	if (fontFaces[i] == (FT_Face)-1) return 0;
 
-	FT_Face face = (FT_Face)mpFontList[i].face;
+	FT_Face face = fontFaces[i];
 	FT_UInt index = FT_Get_Char_Index(face, (FT_ULong)unicode);
 	if (!index) return 0;
 
-	FT_Load_Glyph(face, index, FT_LOAD_RENDER | mpFontList[i].load_flags);
+	FT_Load_Glyph(face, index, FT_LOAD_RENDER | fontFlags[i]);
 	FT_Bitmap &bitmap = face->glyph->bitmap;
-	
+
 	u32 x, y, w, h, nx, ny, nw, nh;
 	x = y = 0;
 	w = nw = bitmap.width;
 	h = nh = bitmap.rows;
-	Screen::rotateRect(x, y, nw, nh);
+	Screen::instance()->rotateRect(x, y, nw, nh);
 
 	Glyph *glyph = (Glyph *)new u8[OFFSET(Glyph, pixmap) + nw * nh];
 	glyph->left = face->glyph->metrics.horiBearingX >> 6;
 	glyph->top = mHeight - 1 + (face->size->metrics.descender >> 6) - (face->glyph->metrics.horiBearingY >> 6);
-	glyph->advance = face->glyph->metrics.horiAdvance >> 6;
 	glyph->width = face->glyph->metrics.width >> 6;
 	glyph->height = face->glyph->metrics.height >> 6;
 	glyph->pitch = nw;
-	glyph->isbitmap = false;
 
 	u8 *buf = bitmap.buffer;
 	for (y = 0; y < h; y++, buf += bitmap.pitch) {
 		for (x = 0; x < w; x++) {
 			nx = x, ny = y;
-			Screen::rotatePoint(w, h, nx, ny);
+			Screen::instance()->rotatePoint(w, h, nx, ny);
 
-			glyph->pixmap[ny * nw + nx] = 
+			glyph->pixmap[ny * nw + nx] =
 				(bitmap.pixel_mode == FT_PIXEL_MODE_MONO) ? ((buf[(x >> 3)] & (0x80 >> (x & 7))) ? 0xff : 0) : buf[x];
 		}
 	}
-	
-	glyphCache[unicode]	= glyph;
+
+	glyphCache[unicode] = glyph;
 	return glyph;
 }
