@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -24,16 +24,15 @@
 #include "type.h"
 #include "instance.h"
 
-#define W(col) (Font::instance()->width() * (col))
-#define H(row) (Font::instance()->height() * (row))
+#define FW(cols) (Font::instance()->width() * (cols))
+#define FH(rows) (Font::instance()->height() * (rows))
 
 class Font {
 	DECLARE_INSTANCE(Font)
 public:
 	struct Glyph {
 		s16 pitch, width, height;
-		s16 left, top, advance;
-		bool isbitmap;
+		s16 left, top;
 		u8 pixmap[0];
 	};
 
@@ -44,26 +43,10 @@ public:
 	u32 height() {
 		return mHeight;
 	}
-	bool isMonospace() {
-		return mMonospace;
-	}
+	void showInfo(bool verbose);
 
 private:
-	struct FontRec {
-		void *pattern;
-		void *face;
-		int load_flags;
-	};
-
-	Font(FontRec *fonts, u32 num, void *unicover);
-	void openFont(u32 index);
-	int fontIndex(u32 unicode);
-
-	void *mpUniCover;
-	FontRec *mpFontList;
-	u32 mFontNum;
 	u32 mWidth, mHeight;
-	bool mMonospace;
 };
 
 #endif
