@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -45,8 +45,15 @@ const s8 *IoPipe::localCodec()
 		// The output character set of gettext is, by default, the value of nl_langinfo(CODESET),
 		// which depends on the LC_CTYPE part of the current locale.
 		setlocale(LC_CTYPE, "");
-		strncpy(local_codec, nl_langinfo(CODESET), sizeof(local_codec) - 1);
+
+		s8 *locale = setlocale(LC_CTYPE, "");
+		if (!locale || !strcmp(locale, "C")) {
+			strcpy(local_codec, "UTF-8");
+		} else {
+			strncpy(local_codec, nl_langinfo(CODESET), sizeof(local_codec) - 1);
+		}
 	}
+
 	return local_codec;
 }
 
@@ -67,7 +74,7 @@ IoPipe::~IoPipe()
 	}
 
 	if (mCodecRead) {
-		 iconv_close((iconv_t)mCodecRead);
+		iconv_close((iconv_t)mCodecRead);
 	}
 
 	if (mCodecWrite) {
@@ -81,7 +88,7 @@ void IoPipe::setFd(s32 fd)
 
 	if (mFd != -1) {
 		IoDispatcher::instance()->removeIoSource(this, true);
-		close(mFd);		
+		close(mFd);
 	}
 
 	mFd = fd;
@@ -94,6 +101,7 @@ void IoPipe::setFd(s32 fd)
 	}
 }
 
+
 void IoPipe::setCodec(const s8 *up, const s8 *down)
 {
 	if (!up || !down) return;
@@ -124,12 +132,13 @@ void IoPipe::ready(bool isread)
 	if (!isread) return;
 
 	s8 buf[BUF_SIZE];
-
 	s32 len = read(mFd, buf + mBufLenRead, sizeof(buf) - mBufLenRead);
-	
-	if (len == -1) {
-		if (errno == EBADF) {
-			ioEnd();
+
+	if (!len) {
+		ioError(true, 0); // end of file
+	} else if (len == -1) {
+		if (errno != EAGAIN && errno != EINTR) {
+			ioError(true, errno);
 		}
 	} else if (len > 0) {
 		if (mBufLenRead) {
@@ -225,11 +234,10 @@ void IoPipe::writeIo(s8 *buf, u32 len)
 				if (num--) {
 					timespec tm = { 0, 200 * 1000000UL };
 					nanosleep(&tm, 0);
-				} else 
-					break;
-			} else if (errno != EINTR) {
-				if (errno == EBADF || errno == EPIPE) {
-					ioEnd();
+				} else break;
+			} else {
+				if (errno != EINTR) {
+					ioError(false, errno);
 				}
 				break;
 			}
