@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -51,8 +51,10 @@ protected:
 	~Shell();
 
 	void resize(u16 w, u16 h);
-	void createChildProcess();
-	virtual void initChildProcess() {}
+	void createShellProcess(s8 **command);
+	s32 shellProcessId() { return mPid; }
+
+	virtual void initShellProcess() {}
 	virtual void readyRead(s8 *buf, u32 len);
 
 private:
@@ -84,7 +86,7 @@ private:
 
 		s8 *text;
 	} mSelText;
-	
+
 	struct TextSelection {
 		TextSelection() {
 			selecting = color_inversed = false;
