@@ -26,6 +26,8 @@
 
 class Mouse : public IoPipe {
 	DECLARE_INSTANCE(Mouse)
+public:
+	void switchVc(bool enter);
 private:
 	virtual void readyRead(s8 *buf, u32 len);
 
